#
# Copyright (C) 2018 Martin Owens
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3.0 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library.
#
"""
Provide some utilities to tests
"""
from logging import Handler, getLogger, root
from collections import defaultdict

# FLAG: do not report failures from here in tracebacks
# pylint: disable=invalid-name
__unittest = True

class LoggingRecorder(Handler):
    """Record any logger output for testing"""
    def __init__(self, *args, **kwargs):
        self.logs = defaultdict(list)
        super(LoggingRecorder, self).__init__(*args, **kwargs)

    def __getitem__(self, name):
        return self.logs[name.upper()]

    def emit(self, record):
        """Save the log message to the right level"""
        # We have no idea why record's getMessage is prefixed
        msg = str(record.getMessage())
        if msg.startswith('u"'):
            msg = msg[1:]
        if msg and (msg[0] == msg[-1] and msg[0] in '"\''):
            msg = msg[1:-1]
        self[record.levelname].append(msg)
        return True


class LoggingMixin(object):
    """Provide logger capture"""
    log_name = None

    def setUp(self):
        """Make a fresh logger for each test function"""
        super(LoggingMixin, self).setUp()
        named = getLogger(self.log_name)
        for handler in root.handlers[:]:
            root.removeHandler(handler)
        for handler in named.handlers[:]:
            named.removeHandler(handler)
        self.log_handler = LoggingRecorder(level='DEBUG')
        named.addHandler(self.log_handler)

    def tearDown(self):
        """Warn about untested logs"""
        for level in self.log_handler.logs:
            for msg in self.log_handler[level]:
                raise ValueError("Uncaught log: {}: {}\n".format(level, msg))

    def assertLog(self, level, msg):
        """Checks that the logger has emitted the given log"""
        logs = self.log_handler[level]
        self.assertTrue(logs, 'Logger hasn\'t emitted "{}"'.format(msg))
        if len(logs) == 1:
            self.assertEqual(msg, logs[0])
        else:
            self.assertIn(msg, logs)
        logs.remove(msg)

    def assertNoLog(self, level, msg):
        """Checks that the logger has NOT emitted the given log"""
        self.assertNotIn(msg, self.log_handler[level])



#
# Copyright 2016, Martin Owens <doctormo@gmail.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3.0 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library.
#
"""
The crontabs manager will list all available crontabs on the system.
"""

import os
import sys
import pwd
import itertools

from os import stat, access, X_OK
from pwd import getpwuid
from crontab import CronTab

class UserSpool(list):
    """Generates all user crontabs, yields both owned and abandoned tabs"""
    def __init__(self, loc, tabs=None):
        for username in self.listdir(loc):
            tab = self.generate(loc, username)
            if tab:
                self.append(tab)
        if not self:
            tab = CronTab(user=True)
            if tab:
                self.append(tab)

    def listdir(self, loc):
        try:
            return os.listdir(loc)
        except OSError:
            return []

    def get_owner(self, path):
        """Returns user file at path"""
        try:
            return getpwuid(stat(path).st_uid).pw_name
        except KeyError:
            return

    def generate(self, loc, username):
        path = os.path.join(loc, username)
        if username != self.get_owner(path):
            # Abandoned crontab pool entry!
            return CronTab(tabfile=path)
        return CronTab(user=username)


class SystemTab(list):
    """Generates all system tabs"""
    def __init__(self, loc, tabs=None):
        if os.path.isdir(loc):
            for item in os.listdir(loc):
                if item[0] == '.':
                    continue
                path = os.path.join(loc, item)
                self.append(CronTab(user=False, tabfile=path))
        elif os.path.isfile(loc):
            self.append(CronTab(user=False, tabfile=loc))


class AnaCronTab(list):
    """Attempts to digest anacron entries (if possible)"""
    def __init__(self, loc, tabs=None):
        if tabs and os.path.isdir(loc):
            self.append(CronTab(user=False))
            jobs = list(tabs.all.find_command(loc))
            if jobs:
                for item in os.listdir(loc):
                    self.add(loc, item, jobs[0])
                jobs[0].delete()

    def add(self, loc, item, anajob):
        path = os.path.join(loc, item)
        if item in ['0anacron'] or item[0] == '.' or not access(path, X_OK):
            return
        job = self[0].new(command=path, user=anajob.user)
        job.set_comment('Anacron %s' % loc.split('.')[-1])
        job.setall(anajob)
        return job


# Files are direct, directories are listed (recursively)
KNOWN_LOCATIONS = [
  # Known linux locations (Debian, RedHat, etc)
  (UserSpool, '/var/spool/cron/crontabs/'),
  (SystemTab, '/etc/crontab'),
  (SystemTab, '/etc/cron.d/'),
  # Anacron digestion (we want to know more)
  (AnaCronTab, '/etc/cron.hourly'),
  (AnaCronTab, '/etc/cron.daily'),
  (AnaCronTab, '/etc/cron.weekly'),
  (AnaCronTab, '/etc/cron.monthly'),
  # Known MacOSX locations
   # None
  # Other (windows, bsd)
   # None
]

class CronTabs(list):
    """Singleton dictionary of all detectable crontabs"""
    _all = None
    _self = None

    def __new__(cls, *args, **kw):
        if not cls._self:
            cls._self = super(CronTabs, cls).__new__(cls, *args, **kw)
        return cls._self

    def __init__(self):
        if not self:
            for loc in KNOWN_LOCATIONS:
                self.add(*loc)

    def add(self, cls, *args):
        for tab in cls(*args, tabs=self):
            self.append(tab)
            self._all = None

    @property
    def all(self):
        """Return a CronTab object with all jobs (read-only)"""
        if self._all is None:
            self._all = CronTab(user=False)
            for tab in self:
                for job in tab:
                    if job.user is None:
                        job.user = tab.user or 'unknown'
                    self._all.append(job)
        return self._all




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 Martin Owens
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3.0 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library.
#
"""
Test crontab use of UTF-8 filenames and strings
"""

import os
import sys

import locale
import unittest
from crontab import CronTab, PY3
try:
    from test import test_support
except ImportError:
    from test import support as test_support

TEST_DIR = os.path.dirname(__file__)

if PY3:
    unicode = str

content = """
*/4 * * * * ůțƒ_command # ůțƒ_comment
"""
filename = os.path.join(TEST_DIR, 'data', 'output-ůțƒ-8.tab')

class Utf8TestCase(unittest.TestCase):
    """Test basic functionality of crontab."""
    def setUp(self):
        self.crontab = CronTab(tab=content)

    def test_01_input(self):
        """Read UTF-8 contents"""
        self.assertTrue(self.crontab)

    def test_02_write(self):
        """Write/Read UTF-8 Filename"""
        self.assertEqual(locale.getpreferredencoding(), 'UTF-8')
        self.crontab.write(filename)
        crontab = CronTab(tabfile=filename)
        self.assertTrue(crontab)
        with open(filename, "r") as fhl:
            self.assertEqual(content, fhl.read())
        os.unlink(filename)

    def test_04_command(self):
        """Read Command String"""
        self.assertEqual(self.crontab[0].command, u"ůțƒ_command")

    def test_05_comment(self):
        """Read Comment String"""
        self.assertEqual(self.crontab[0].comment, u'ůțƒ_comment')

    def test_06_unicode(self):
        """Write New via Unicode"""
        job = self.crontab.new(command=u"ůțƒ_command", comment=u'ůțƒ_comment')
        self.assertEqual(job.command, u"ůțƒ_command")
        self.assertEqual(job.comment, u"ůțƒ_comment")
        self.crontab.render()

    def test_07_utf8(self):
        """Write New via UTF-8"""
        job = self.crontab.new(command=b'\xc5\xaf\xc8\x9b\xc6\x92_command',
                               comment=b'\xc5\xaf\xc8\x9b\xc6\x92_comment')
        self.assertEqual(self.crontab.render(), u"""
*/4 * * * * ůțƒ_command # ůțƒ_comment

* * * * * ůțƒ_command # ůțƒ_comment
""")
        self.assertEqual(type(job.command), unicode)
        self.assertEqual(type(job.comment), unicode)

    def test_08_utf8_str(self):
        """Test UTF8 (non unicode) strings"""
        self.crontab[0].command = '￡１２'
        self.crontab[0].comment = '￼𝗔𝗕𝗖𝗗'
        self.assertEqual(self.crontab.render(), u"""
*/4 * * * * ￡１２ # ￼𝗔𝗕𝗖𝗗
""")

    def test_09_utf8_again(self):
        """Test Extra UTF8 input"""
        filename = os.path.join(TEST_DIR, 'data', 'utf_extra')
        with open(filename + '.in', 'w') as fhl:
            fhl.write('# 中文\n30 2 * * * source /etc/profile\n30 1 * * * source /etc/profile')

        try:
            cron = CronTab(tabfile=filename + '.in')
        finally:
            os.unlink(filename + '.in')

        cron.write(filename + '.out')
        if os.path.isfile(filename + '.out'):
            os.unlink(filename + '.out')

if __name__ == '__main__':
    test_support.run_unittest(
       Utf8TestCase,
    )



#!/usr/bin/env python
#
# Copyright (C) YEAR Martin Owens
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3.0 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library.
#
"""
Test the cron log extention with a test syslog example data file.
"""

import os
import sys
from datetime import datetime, date

import unittest
from crontab import CronTab
from cronlog import CronLog, LogReader
try:
    from test import test_support
except ImportError:
    from test import support as test_support

TEST_DIR = os.path.dirname(__file__)

INITAL_TAB = """
* * * * * userscript &> /dev/null
* * * * * rootscript &> /dev/null
* * * * * shadowscript &> /dev/null
"""

ROOT_PIDS = ['16592', '16574', '16552', '16522', '16514', '16489']
SHAD_PIDS = ['16551', '16497']
USER_PIDS = ['16591', '16588', '16573', '16569', '16554', '16539', '16523',\
            '16519','16513','16496','16490']

YEAR = date.today().year

ROOT_DATES = [
    datetime(YEAR, 4, 4, 21, 34, 1),
    datetime(YEAR, 4, 4, 21, 32, 1),
    datetime(YEAR, 4, 4, 21, 30, 1),
    datetime(YEAR, 4, 4, 21, 28, 1),
    datetime(YEAR, 4, 4, 21, 26, 1),
    datetime(YEAR, 4, 4, 21, 24, 1),
  ]
SHAD_DATES = [
    datetime(YEAR, 4, 4, 21, 30, 1),
    datetime(YEAR, 4, 4, 21, 25, 1),
  ]
USER_DATES = [
    datetime(YEAR, 4, 4, 21, 34, 1),
    datetime(YEAR, 4, 4, 21, 33, 2),
    datetime(YEAR, 4, 4, 21, 32, 1),
    datetime(YEAR, 4, 4, 21, 31, 1),
    datetime(YEAR, 4, 4, 21, 30, 1),
    datetime(YEAR, 4, 4, 21, 29, 1),
    datetime(YEAR, 4, 4, 21, 28, 1),
    datetime(YEAR, 4, 4, 21, 27, 1),
    datetime(YEAR, 4, 4, 21, 26, 1),
    datetime(YEAR, 4, 4, 21, 25, 1),
    datetime(YEAR, 4, 4, 21, 24, 1),
  ]
LONG_LINE = 'A really long line which can test log lines ability to put two bits together'
READ_LINE = [ 'The End', 'Sickem', '2', '9', 'First Line', LONG_LINE ]

class BasicTestCase(unittest.TestCase):
    """Test basic functionality of crontab."""
    def setUp(self):
        self.log = os.path.join(TEST_DIR, 'data', 'test.log')
        self.crontab = CronTab(tab=INITAL_TAB, log=self.log)

    def test_00_logreader(self):
        """Log Reader"""
        lines = READ_LINE[:]
        reader = LogReader(os.path.join(TEST_DIR, 'data', 'basic.log'), mass=50)
        for line in reader:
            self.assertEqual(line.strip(), lines.pop(0))

    def test_01_cronreader(self):
        """Cron Log Lines"""
        with self.crontab.log as log:
            lines = list(log.readlines())
        self.assertEqual(len(lines), 32)
        self.assertEqual(lines[0][1], "Apr  4 21:34:01 servername CRON[16592]"
                                      ": (root) CMD (rootscript &> /dev/null)")
        self.assertEqual(lines[15][1], "Apr  4 21:28:31 servername NOTCRON"
                                      ": that these are ignored")
        self.assertEqual(lines[-1][1], "Apr  4 21:24:01 servername CRON[16490]"
                                      ": (user) CMD (userscript &> /dev/null)")

    def test_01_iter(self):
        """Iterate directly over log"""
        with self.crontab.log as log:
            self.assertEqual(len(list(log)), 8)

    def test_02_cronlog(self):
        """Cron Log Items"""
        entries = list(CronLog(os.path.join(TEST_DIR, 'data', 'test.log')))
        self.assertEqual(len(entries), 19)
        self.assertEqual(entries[0]['pid'], "16592")
        self.assertEqual(entries[3]['pid'], "16574")
        self.assertEqual(entries[-1]['pid'], "16490")

    def test_03_crontab(self):
        """Cron Tab Items"""
        entries = list(self.crontab.log)
        self.assertEqual(len(entries), 8)
        self.assertEqual(entries[0]['pid'], "16592")
        self.assertEqual(entries[3]['pid'], "16551")
        self.assertEqual(entries[-1]['pid'], "16489")

    def test_04_root(self):
        """Cron Job Items"""
        pids, dates = ROOT_PIDS[:], ROOT_DATES[:]
        job = list(self.crontab.find_command('rootscript'))[0]
        for log in job.log:
            self.assertEqual(log['pid'], pids.pop(0))
            self.assertEqual(log['date'], dates.pop(0))
        self.assertEqual(pids, [])

    def tst_05_shadow(self):
        """Seperate Job Items"""
        pids, dates = SHAD_PIDS[:], SHAD_DATES[:]
        job = self.crontab.find_command('shadowscript')[0]
        for log in job.log:
            self.assertEqual(log['pid'], pids.pop(0))
            self.assertEqual(log['date'], dates.pop(0))
        self.assertEqual(pids, [])

    def tst_06_user(self):
        """Seperate User Crontab"""
        pids, dates = USER_PIDS[:], User_DATES[:]
        self.crontab.user = 'user'
        job = self.crontab.find_command('userscript')[0]
        for log in job.log:
            self.assertEqual(log['pid'], pids.pop(0))
            self.assertEqual(log['date'], dates.pop(0))
        self.assertEqual(pids, [])

    def test_08_readerror(self):
        """Cron Log Error"""
        self.crontab.log.pipe = False
        with self.assertRaises(IOError):
            list(self.crontab.log.readlines())


if __name__ == '__main__':
    test_support.run_unittest(
       BasicTestCase,
    )



#!/usr/bin/env python
#
# Copyright (C) 2015 Martin Owens
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3.0 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library.
#
"""
System cron is prefixed with the username the process should run under.
"""

import os
import sys

sys.path.insert(0, '../')

import unittest
from crontab import CronTab
try:
    from test import test_support
except ImportError:
    from test import support as test_support

TEST_FILE = os.path.join(os.path.dirname(__file__), 'data', 'crontab')
INITIAL_TAB = """
*/30 * * * * palin one_cross_each
*/30 * * * * matias\ttab_user_error
"""

class SystemCronTestCase(unittest.TestCase):
    """Test vixie cron user addition."""
    def setUp(self):
        self.crontab = CronTab(tab=INITIAL_TAB, user=False)

    def test_00_repr(self):
        """System crontab repr"""
        self.assertEqual(repr(self.crontab), "<Unattached System CronTab>")

    def test_01_read(self):
        """Read existing command"""
        job = self.crontab[0]
        self.assertEqual(job.user, 'palin')
        self.assertEqual(job.command, 'one_cross_each')
        job = self.crontab[1]
        self.assertEqual(job.user, 'matias')
        self.assertEqual(job.command, 'tab_user_error')
        self.assertEqual(len(self.crontab), 2)

    def test_02_new(self):
        """Create a new job"""
        job = self.crontab.new(command='release_brian', user='pontus')
        self.assertEqual(job.user, 'pontus')
        self.assertEqual(job.command, 'release_brian')
        self.assertEqual(str(self.crontab), INITIAL_TAB.replace('\t', ' ') + """
* * * * * pontus release_brian
""")

    def test_03_new_tab(self):
        """Create a new system crontab"""
        tab = CronTab(user=False)
        tab.env['SHELL'] = '/usr/bin/roman'
        job = tab.new(command='release_bwian', user='pontus')
        self.assertEqual(str(tab), """SHELL=/usr/bin/roman
* * * * * pontus release_bwian
""")

    def test_04_failure(self):
        """Fail when no user"""
        with self.assertRaises(ValueError):
            self.crontab.new(command='im_brian')
        cron = self.crontab.new(user='user', command='no_im_brian')
        cron.user = None
        with self.assertRaises(ValueError):
            cron.render()

    def test_05_remove(self):
        """Remove the user flag"""
        self.crontab._user = None
        self.assertEqual(str(self.crontab), """
*/30 * * * * one_cross_each
*/30 * * * * tab_user_error
""")
        self.crontab.new(command='now_go_away')


    def test_06_comments(self):
        """Comment with six parts parses successfully"""
        crontab = CronTab(user=False, tab="""
#a system_comment that has six parts_will_fail_to_parse
        """)

    def test_07_recreation(self):
        """Input doesn't change on save"""
        crontab = CronTab(user=False, tab="* * * * * user command")
        self.assertEqual(str(crontab), "* * * * * user command\n")
        crontab = CronTab(user=False, tab="* * * * * user command\n")
        self.assertEqual(str(crontab), "* * * * * user command\n")

    def test_09_resaving(self):
        """Cycle rendering to show no changes"""
        for i in range(10):
            self.crontab = CronTab(tab=str(self.crontab))

        self.assertEqual(INITIAL_TAB.replace('\t', ' '), str(self.crontab))

    def test_10_system_file(self):
        """Load system crontab from a file"""
        crontab = CronTab(user=False, tabfile=TEST_FILE)
        self.assertEqual(repr(crontab), "<System CronTab '%s'>" % TEST_FILE)

if __name__ == '__main__':
    test_support.run_unittest(SystemCronTestCase)



#!/usr/bin/env python
#
# Copyright (C) 2011 Martin Owens, 2019 Jordan Miller
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3.0 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library.
#
"""
Test crontab context.
"""

import os
import sys

sys.path.insert(0, '../')

import unittest
from crontab import CronTab
try:
    from test import test_support
except ImportError:
    from test import support as test_support

INITAL_TAB = """
* * * JAN     SAT          enums
* * * MAR-APR *            ranges
* * * *       MON,FRI,WED  multiples
* * * *       *            com1 # Comment One
* * * *       *            com2 # Comment One
"""

RESULT_TAB = """
* * * JAN SAT enums
* * * MAR-APR * ranges
* * * * MON,WED,FRI multiples
* * * * * com1 # Comment One
* * * * * com2 # Comment One
"""

class ContextTestCase(unittest.TestCase):
    """Test basic functionality of crontab."""
    def setUp(self):
        self.crontab = CronTab(tab=INITAL_TAB)

    def test_01_context(self):
        """context matches non-context"""
        self.crontab.write()
        results = RESULT_TAB.split('\n')
        for line_no, line in enumerate(self.crontab.intab.split('\n')):
            self.assertEqual(str(line), results[line_no])
        with CronTab(tab=INITAL_TAB) as crontab_context:
            self.crontab_context = crontab_context
        for line_no, line in enumerate(self.crontab_context.intab.split('\n')):
            self.assertEqual(str(line), results[line_no])

if __name__ == '__main__':
    test_support.run_unittest(
       ContextTestCase,
    )



#!/usr/bin/env python
#
# Copyright (C) 2013 Martin Owens
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3.0 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library.
#
"""
Test frequency calculations
"""

import os
import sys

sys.path.insert(0, '../')

import unittest
from crontab import CronTab, PY3
try:
    from test import test_support
except ImportError:
    from test import support as test_support

if PY3:
    unicode = str

START_TAB = """
"""

class FrequencyTestCase(unittest.TestCase):
    """Test basic functionality of crontab."""
    def setUp(self):
        self.crontab = CronTab(tab=START_TAB.strip())
        self.job = self.crontab.new(command='freq')

    def test_00_slice_frequency(self):
        """Each Slice Frequency"""
        # Make sure some of these overlap
        self.job.setall("13-16,14-15 6,*/3 2,3,4 * *")
        self.assertEqual(len(self.job[0]), 4)
        self.assertEqual(len(self.job[1]), 8)
        self.assertEqual(len(self.job[2]), 3)
        self.assertEqual(len(self.job[3]), 12)
        self.assertEqual(len(self.job[4]), 7)

    def test_01_per_day(self):
        """Frequency per Day"""
        self.job.setall("21-29 10-14 * * *")
        self.assertEqual(len(self.job[0]), 9)
        self.assertEqual(len(self.job[1]), 5)
        self.assertEqual(self.job.frequency_per_day(), 45)

    def test_02_days_per_year(self):
        """Frequency in Days per Year"""
        self.job.setall("* * * * *")
        self.assertEqual(self.job.frequency_per_year(year=2010), 365)
        self.assertEqual(self.job.frequency_per_year(year=2012), 366)
        self.job.setall("1 1 11-20 1,4,6,8 0,6")
        self.assertEqual(len(self.job[2]), 10)
        self.assertEqual(len(self.job[3]), 4)
        self.assertEqual(len(self.job[4]), 2)
        self.assertEqual(self.job.frequency_per_year(year=2013), 11)
        self.assertEqual(self.job.frequency_per_year(year=2010), 12)

    def test_03_job(self):
        """Once Yearly"""
        self.job.setall("0 0 1 1 *")
        self.assertEqual(self.job.frequency(year=2010), 1)

    def test_04_twice(self):
        """Twice Yearly"""
        self.job.setall("0 0 1 1,6 *")
        self.assertEqual(self.job.frequency(year=2010), 2)

    def test_05_thrice(self):
        """Thrice Yearly"""
        self.job.setall("0 0 1 1,3,6 *")
        self.assertEqual(self.job.frequency(year=2010), 3)

    def test_06_quart(self):
        """Four Yearly"""
        self.job.setall("0 0 1 */3 *")
        self.assertEqual(self.job.frequency(year=2010), 4)

    def test_07_monthly(self):
        """Once a month"""
        self.job.setall("0 0 1 * *")
        self.assertEqual(self.job.frequency(year=2010), 12)

    def test_08_six_monthly(self):
        """Six a month"""
        self.job.setall("0 0 1,2,3,4,5,6 * *")
        self.assertEqual(self.job.frequency(year=2010), 72)

    def test_09_every_day(self):
        """Every Day"""
        self.job.setall("0 0 * * *")
        self.assertEqual(self.job.frequency(year=2010), 365)

    def test_10_every_hour(self):
        """Every Hour"""
        self.job.setall("0 * * * *")
        self.assertEqual(self.job.frequency(year=2010), 8760)

    def test_11_every_other_hour(self):
        """Every Other Hour"""
        self.job.setall("0 */2 * * *")
        self.assertEqual(self.job.frequency(year=2010), 4380)

    def test_12_every_minute(self):
        """Every Minute"""
        self.job.setall("* * * * *")
        self.assertEqual(self.job.frequency(year=2010), 525600)

    def test_13_enum(self):
        """Enumerations"""
        self.job.setall("0 0 * * MON-WED")
        self.assertEqual(self.job.frequency(year=2010), 156)
        self.job.setall("0 0 * JAN-MAR *")
        self.assertEqual(self.job.frequency(year=2010), 90)

    def test_14_all(self):
        """Test Maximum"""
        self.job.setall("* * * * *")
        self.assertEqual(self.job.frequency(2010), 525600)
        self.assertEqual(self.job.frequency_per_year(year=2010), 365)
        self.assertEqual(self.job.frequency_per_day(), 1440)
        self.job.setall("*")
        self.assertEqual(self.job.frequency_per_day(), 1440)
        self.assertEqual(self.job.frequency_per_year(year=2010), 365)
        self.assertEqual(self.job.frequency(2010), 525600)

    def test_15_compare(self):
        """Compare Times"""
        job = self.crontab.new(command='match')
        job.setall("*/2 * * * *")
        self.assertEqual(job.slices, "*/2 * * * *")
        self.assertEqual(job.slices, ["*/2"])
        self.assertLess(job, ["*"])
        self.assertGreater(job, "*/3")

    def test_16_frequency_per_hour(self):
        """Count per hour"""
        job = self.crontab.new(command='per_hour')
        job.setall("*/2 * * * *")
        self.assertEqual(job.frequency_per_hour(), 30)

if __name__ == '__main__':
    test_support.run_unittest(
       FrequencyTestCase,
    )



#!/usr/bin/env python
#
# Copyright (C) 2016 Martin Owens <doctormo@gmail.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3.0 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library.
#
"""
Test crontabs listing all available crontabs.
"""

import os
import sys

import unittest
import crontab
import crontabs

try:
    from test import test_support
except ImportError:
    from test import support as test_support

TEST_DIR = os.path.dirname(__file__)

class fake_getpwuid(object):
    def __init__(self, st):
        self.pw_name = st
        if st == 'error':
            raise KeyError("Expected Error")

class fake_stat(object):
    def __init__(self, path):
        self.st_uid = os.path.basename(path).split('_')[0]

crontabs.getpwuid = fake_getpwuid
crontabs.stat = fake_stat

class AnacronSourceTab(list):
    """This enables the anacron detection"""
    slices = "56 4 * * 4"
    def __init__(self, tabs=None):
        self.append(crontab.CronTab(user=False, tab="%s root anacron_cmd %s" \
            % (self.slices, os.path.join(TEST_DIR, 'data', 'anacron'))))

crontabs.KNOWN_LOCATIONS = [
    (AnacronSourceTab,),
    (crontabs.UserSpool, os.path.join(TEST_DIR, 'data', 'spool')),
    (crontabs.UserSpool, os.path.join(TEST_DIR, 'data', 'bad_spool')),
    (crontabs.SystemTab, os.path.join(TEST_DIR, 'data', 'crontab')),
    (crontabs.SystemTab, os.path.join(TEST_DIR, 'data', 'crontabs')),
    (crontabs.AnaCronTab, os.path.join(TEST_DIR, 'data', 'anacron')),
]

crontab.CRONCMD = "%s %s" % (sys.executable, os.path.join(TEST_DIR, 'data', 'crontest'))

class CronTabsTestCase(unittest.TestCase):
    """Test use documentation in crontab."""
    def setUp(self):
        self.tabs = crontabs.CronTabs()

    def assertInTabs(self, command, *users):
        jobs = list(self.tabs.all.find_command(command))
        self.assertEqual(len(jobs), len(users))
        users = sorted([job.user for job in jobs])
        self.assertEqual(users, sorted(users))
        return jobs

    def assertNotInTabs(self, command):
        return self.assertInTabs(command)

    def test_05_spool(self):
        """Test a user spool"""
        self.assertInTabs('do_vlog_brothers', 'hgreen', 'jgreen')

    def test_06_bad_spool(self):
        """Test no access to spool (non-root)"""
        # IMPORTANT! This is testing the fact that the bad-spool will load
        # the user's own crontab, in this instance this is 'user' from the
        # crontest script. This tab is already loaded by the previous User
        # spool and so we expect to find two of them.
        self.assertInTabs('user_command', 'user', 'user')

    def test_10_crontab_dir(self):
        """Test crontabs loaded from system directory"""
        self.assertInTabs('baggins', 'bilbo', 'frodo', 'plastic')

    def test_11_crontab_file(self):
        """Test a single crontab file loaded from system tab"""
        self.assertInTabs('parker', 'driver', 'peter')

    def test_20_anacron(self):
        """Anacron digested"""
        self.assertNotInTabs('anacron_cmd')
        jobs = self.assertInTabs('an_command.sh', 'root')
        self.assertEqual(str(jobs[0].slices), AnacronSourceTab.slices)
        self.assertNotInTabs('not_command.txt')
        

if __name__ == '__main__':
    test_support.run_unittest(CronTabsTestCase)




#
# Copyright 2013, Martin Owens <doctormo@gmail.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3.0 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library.
#
"""
Access logs in known locations to find information about them.
"""

import os
import re
import codecs
import platform

PY3 = platform.python_version()[0] == '3'

if PY3:
     # pylint: disable=W0622
     unicode = str
     basestring = str

from dateutil import parser as dateparse

MATCHER = r'(?P<date>\w+ +\d+ +\d\d:\d\d:\d\d) (?P<host>\w+) ' + \
        r'CRON\[(?P<pid>\d+)\]: \((?P<user>\w+)\) CMD \((?P<cmd>.*)\)'

class LogReader(object):
    """Opens a Log file, reading backwards and watching for changes"""
    def __init__(self, filename, mass=4096):
        self.filename = filename
        self.mass = mass
        self.size = -1
        self.read = -1
        self.pipe = None

    def __enter__(self):
        self.size = os.stat(self.filename)[6]
        self.pipe = codecs.open(self.filename, 'r', encoding='utf-8')
        return self

    def __exit__(self, error_type, value, traceback):
        self.pipe.close()

    def __iter__(self):
        if self.pipe is None:
            with self as reader:
                for (offset, line) in reader.readlines():
                    yield line
        else:
            for (offset, line) in self.readlines():
                yield line

    def readlines(self, until=0):
        """Iterator for reading lines from a file backwards"""
        if not self.pipe or self.pipe.closed:
            raise IOError("Can't readline, no opened file.")
        # Always seek to the end of the file, this accounts for file updates
        # that happen during our running process.
        location = self.size
        halfline = ''

        while location > until:
            location -= self.mass
            mass = self.mass
            if location < 0:
                mass = self.mass + location
                location = 0
            self.pipe.seek(location)
            line = self.pipe.read(mass) + halfline
            data = line.split('\n')
            if location != 0:
                halfline = data.pop(0)
            loc = location + mass
            data.reverse()
            for line in data:
                if line.strip() == '':
                    continue
                yield (loc, line)
                loc -= len(line)



class CronLog(LogReader):
    """Use the LogReader to make a Cron specific log reader"""
    def __init__(self, filename='/var/log/syslog', user=None):
        LogReader.__init__(self, filename)
        self.user = user

    def for_program(self, command):
        """Return log entries for this specific command name"""
        return ProgramLog(self, command)

    def __iter__(self):
        for line in super(CronLog, self).__iter__():
            match = re.match(MATCHER, unicode(line))
            datum = match and match.groupdict()
            if datum and (not self.user or datum['user'] == self.user):
                datum['date'] = dateparse.parse(datum['date'])
                yield datum


class ProgramLog(object):
    """Specific log control for a single command/program"""
    def __init__(self, log, command):
        self.log = log
        self.command = command

    def __iter__(self):
        for entry in self.log:
            if entry['cmd'] == unicode(self.command):
                yield entry

